@@ -5,24 +5,25 @@ import logging
 import fnmatch
 from threading import Event
 
-# pylint: disable=wildcard-import
 import threading
 
 import zeroconf
 
-from .config import *  # noqa
-from .error import *  # noqa
+from .config import *  # noqa: F403
+from .error import *  # noqa: F403
 from . import socket_client
-from .discovery import (  # noqa
+from .discovery import (  # noqa: F401
     DISCOVER_TIMEOUT,
-    CastListener,
+    CastBrowser,
+    CastListener,  # Deprecated
+    SimpleCastListener,
     discover_chromecasts,
     start_discovery,
     stop_discovery,
 )
 from .dial import get_device_status, DeviceStatus
 from .const import CAST_MANUFACTURERS, CAST_TYPES, CAST_TYPE_CHROMECAST
-from .controllers.media import STREAM_TYPE_BUFFERED  # noqa
+from .controllers.media import STREAM_TYPE_BUFFERED  # noqa: F401
 
 __all__ = ("__version__", "__version_info__", "get_chromecasts", "Chromecast")
 __version_info__ = ("0", "7", "6")
@@ -64,22 +65,22 @@ def get_chromecast_from_host(host, tries=None, retry_wait=None, timeout=None):
 _get_chromecast_from_host = get_chromecast_from_host  # pylint: disable=invalid-name
 
 
-def get_chromecast_from_service(
-    services, zconf, tries=None, retry_wait=None, timeout=None
+def get_chromecast_from_cast_info(
+    cast_info, zconf, tries=None, retry_wait=None, timeout=None
 ):
     """Creates a Chromecast object from a zeroconf service."""
-    # Build device status from the mDNS service name info, this
+    # Build device status from the CastInfo, this
     # information is the primary source and the remaining will be
     # fetched later on.
-    services, uuid, model_name, friendly_name, _, _ = services
-    _LOGGER.debug("_get_chromecast_from_service %s", services)
-    cast_type = CAST_TYPES.get(model_name.lower(), CAST_TYPE_CHROMECAST)
-    manufacturer = CAST_MANUFACTURERS.get(model_name.lower(), "Google Inc.")
+    services = cast_info.services
+    _LOGGER.debug("get_chromecast_from_cast_info %s", services)
+    cast_type = CAST_TYPES.get(cast_info.model_name.lower(), CAST_TYPE_CHROMECAST)
+    manufacturer = CAST_MANUFACTURERS.get(cast_info.model_name.lower(), "Google Inc.")
     device = DeviceStatus(
-        friendly_name=friendly_name,
-        model_name=model_name,
+        friendly_name=cast_info.friendly_name,
+        model_name=cast_info.model_name,
         manufacturer=manufacturer,
-        uuid=uuid,
+        uuid=cast_info.uuid,
         cast_type=cast_type,
     )
     return Chromecast(
@@ -95,7 +96,7 @@ def get_chromecast_from_service(
 
 # Alias for backwards compatibility
 _get_chromecast_from_service = (  # pylint: disable=invalid-name
-    get_chromecast_from_service
+    get_chromecast_from_cast_info
 )
 
 
@@ -116,10 +117,9 @@ def get_listed_chromecasts(
       A list of Chromecast objects matching the criteria,
       or an empty list if no matching chromecasts were found.
       A service browser to keep the Chromecast mDNS data updated. When updates
-      are (no longer) needed, pass the browser object to
-      pychromecast.discovery.stop_discovery().
+      are (no longer) needed, call browser.stop_discovery().
 
-    To only discover chromcast devices wihtout connecting to them, use
+    To only discover chromecast devices without connecting to them, use
     discover_listed_chromecasts instead.
 
     :param friendly_names: A list of wanted friendly names
@@ -134,20 +134,21 @@ def get_listed_chromecasts(
 
     cc_list = {}
 
-    def callback(uuid, name):  # pylint: disable=unused-argument
-        _LOGGER.debug("Found chromecast %s", uuid)
+    def add_callback(uuid, _service):
+        _LOGGER.debug(
+            "Found chromecast %s (%s)", browser.devices[uuid].friendly_name, uuid
+        )
 
         def get_chromecast_from_uuid(uuid):
-            return get_chromecast_from_service(
-                listener.services[uuid],
+            return get_chromecast_from_cast_info(
+                browser.devices[uuid],
                 zconf=zconf,
                 tries=tries,
                 retry_wait=retry_wait,
                 timeout=timeout,
             )
 
-        service = listener.services[uuid]
-        friendly_name = service[3]
+        friendly_name = browser.devices[uuid].friendly_name
         try:
             if uuids and uuid in uuids:
                 if uuid not in cc_list:
@@ -159,14 +160,14 @@ def get_listed_chromecasts(
                 friendly_names.remove(friendly_name)
             if not friendly_names and not uuids:
                 discover_complete.set()
-        except ChromecastConnectionError:  # noqa
+        except ChromecastConnectionError:  # noqa: F405
             pass
 
     discover_complete = Event()
 
-    listener = CastListener(callback)
     zconf = zeroconf_instance or zeroconf.Zeroconf()
-    browser = start_discovery(listener, zconf)
+    browser = CastBrowser(SimpleCastListener(add_callback), zconf)
+    browser.start_discovery()
 
     # Wait for the timeout or found all wanted devices
     discover_complete.wait(discovery_timeout)
@@ -190,17 +191,16 @@ def get_chromecasts(
       A list of Chromecast objects, or an empty list if no matching chromecasts were
       found.
       A service browser to keep the Chromecast mDNS data updated. When updates
-      are (no longer) needed, pass the browser object to
-      pychromecast.discovery.stop_discovery().
+      are (no longer) needed, call browser.stop_discovery().
 
-    To only discover chromcast devices wihtout connecting to them, use
+    To only discover chromecast devices without connecting to them, use
     discover_chromecasts instead.
 
     Parameters tries, timeout, retry_wait and blocking_app_launch controls the
     behavior of the created Chromecast instances.
 
     :param tries: Number of retries to perform if the connection fails.
-                  None for inifinite retries.
+                  None for infinite retries.
     :param timeout: A floating point number specifying the socket timeout in
                     seconds. None means to use the default which is 30 seconds.
     :param retry_wait: A floating point number specifying how many seconds to
@@ -209,26 +209,26 @@ def get_chromecasts(
     :param blocking: If True, returns a list of discovered chromecast devices.
                      If False, triggers a callback for each discovered chromecast,
                      and returns a function which can be executed to stop discovery.
-    :param callback: Callback which is triggerd for each discovered chromecast when
+    :param callback: Callback which is triggered for each discovered chromecast when
                      blocking = False.
     :param zeroconf_instance: An existing zeroconf instance.
     """
     if blocking:
         # Thread blocking chromecast discovery
-        services, browser = discover_chromecasts()
+        devices, browser = discover_chromecasts()
         cc_list = []
-        for service in services:
+        for device in devices:
             try:
                 cc_list.append(
-                    get_chromecast_from_service(
-                        service,
+                    get_chromecast_from_cast_info(
+                        device,
                         browser.zc,
                         tries=tries,
                         retry_wait=retry_wait,
                         timeout=timeout,
                     )
                 )
-            except ChromecastConnectionError:  # noqa
+            except ChromecastConnectionError:  # noqa: F405
                 pass
         return (cc_list, browser)
 
@@ -238,14 +238,14 @@ def get_chromecasts(
 
     known_uuids = set()
 
-    def internal_callback(uuid, name):  # pylint: disable=unused-argument
+    def add_callback(uuid, _service):
         """Called when zeroconf has discovered a new chromecast."""
         if uuid in known_uuids:
             return
         try:
             callback(
-                get_chromecast_from_service(
-                    listener.services[uuid],
+                get_chromecast_from_cast_info(
+                    browser.devices[uuid],
                     zconf=zconf,
                     tries=tries,
                     retry_wait=retry_wait,
@@ -253,12 +253,12 @@ def get_chromecasts(
                 )
             )
             known_uuids.add(uuid)
-        except ChromecastConnectionError:  # noqa
+        except ChromecastConnectionError:  # noqa: F405
             pass
 
-    listener = CastListener(internal_callback)
     zconf = zeroconf_instance or zeroconf.Zeroconf()
-    browser = start_discovery(listener, zconf)
+    browser = CastBrowser(SimpleCastListener(add_callback), zconf)
+    browser.start_discovery()
     return browser
 
 
@@ -274,13 +274,13 @@ class Chromecast:
     :param device: DeviceStatus with initial information for the device.
     :type device: pychromecast.dial.DeviceStatus
     :param tries: Number of retries to perform if the connection fails.
-                  None for inifinite retries.
+                  None for infinite retries.
     :param timeout: A floating point number specifying the socket timeout in
                     seconds. None means to use the default which is 30 seconds.
     :param retry_wait: A floating point number specifying how many seconds to
                        wait between each retry. None means to use the default
                        which is 5 seconds.
-    :param services: A list of mDNS services to try to connect to. If present,
+    :param services: A set of mDNS or host services to try to connect to. If present,
                      parameters host and port are ignored and host and port are
                      instead resolved through mDNS. The list of services may be
                      modified, for example if speaker group leadership is handed
@@ -325,7 +325,7 @@ class Chromecast:
             self.device = get_device_status(host, services, zconf)
 
         if not self.device:
-            raise ChromecastConnectionError(  # noqa
+            raise ChromecastConnectionError(  # noqa: F405
                 "Could not connect to {}:{}".format(host, port or 8009)
             )
 
@@ -363,10 +363,8 @@ class Chromecast:
     def ignore_cec(self):
         """ Returns whether the CEC data should be ignored. """
         return self.device is not None and any(
-            [
-                fnmatch.fnmatchcase(self.device.friendly_name, pattern)
-                for pattern in IGNORE_CEC
-            ]
+            fnmatch.fnmatchcase(self.device.friendly_name, pattern)
+            for pattern in IGNORE_CEC
         )
 
     @property
@@ -486,7 +484,7 @@ class Chromecast:
                         operation in seconds (or fractions thereof). Or None
                         to block forever.
         """
-        if not self.socket_client.isAlive():
+        if not self.socket_client.is_alive():
             self.socket_client.start()
         self.status_event.wait(timeout=timeout)
 
