@@ -14,6 +14,7 @@ APP_SPOTIFY = "CC32E753"
 APP_HOME_ASSISTANT = "B12CE3CA"
 APP_SUPLA = "A41B766D"
 APP_YLEAREENA = "A9BCCB7C"
+APP_BUBBLEUPNP = "3927FA74"
 
 
 def get_possible_app_ids():
@@ -36,7 +37,7 @@ def get_app_config(app_id):
     """ Get specific configuration for 'app_id'. """
     try:
         req = requests.get(
-            ("https://clients3.google.com/" "cast/chromecast/device/app?a={}").format(
+            ("https://clients3.google.com/cast/chromecast/device/app?a={}").format(
                 app_id
             )
         )
