@@ -2,6 +2,7 @@
 Provides a controller for controlling the default media players
 on the Chromecast.
 """
+import abc
 from datetime import datetime
 import logging
 
@@ -9,6 +10,7 @@ from collections import namedtuple
 import threading
 
 from ..config import APP_MEDIA_RECEIVER
+from ..const import MESSAGE_TYPE
 from . import BaseController
 
 STREAM_TYPE_UNKNOWN = "UNKNOWN"
@@ -21,8 +23,6 @@ MEDIA_PLAYER_STATE_PAUSED = "PAUSED"
 MEDIA_PLAYER_STATE_IDLE = "IDLE"
 MEDIA_PLAYER_STATE_UNKNOWN = "UNKNOWN"
 
-MESSAGE_TYPE = "type"
-
 TYPE_EDIT_TRACKS_INFO = "EDIT_TRACKS_INFO"
 TYPE_GET_STATUS = "GET_STATUS"
 TYPE_LOAD = "LOAD"
@@ -32,6 +32,7 @@ TYPE_PAUSE = "PAUSE"
 TYPE_PLAY = "PLAY"
 TYPE_QUEUE_NEXT = "QUEUE_NEXT"
 TYPE_QUEUE_PREV = "QUEUE_PREV"
+TYPE_QUEUE_UPDATE = "QUEUE_UPDATE"
 TYPE_SEEK = "SEEK"
 TYPE_STOP = "STOP"
 
@@ -76,7 +77,6 @@ _LOGGER = logging.getLogger(__name__)
 class MediaStatus:
     """ Class to hold the media status. """
 
-    # pylint: disable=too-many-instance-attributes,too-many-public-methods
     def __init__(self):
         self.current_time = 0
         self.content_id = None
@@ -309,12 +309,19 @@ class MediaStatus:
         return "<MediaStatus {}>".format(info)
 
 
-# pylint: disable=too-many-public-methods
+class MediaStatusListener(abc.ABC):
+    """Listener for receiving media status events."""
+
+    @abc.abstractmethod
+    def new_media_status(self, status: MediaStatus):
+        """Updated media status."""
+
+
 class MediaController(BaseController):
     """ Controller to interact with Google media namespace. """
 
     def __init__(self):
-        super(MediaController, self).__init__("urn:x-cast:com.google.cast.media")
+        super().__init__("urn:x-cast:com.google.cast.media")
 
         self.media_session_id = 0
         self.status = MediaStatus()
@@ -331,7 +338,7 @@ class MediaController(BaseController):
         self.status = MediaStatus()
         self._fire_status_changed()
 
-    def receive_message(self, message, data: dict):
+    def receive_message(self, _message, data: dict):
         """ Called when a media message is received. """
         if data[MESSAGE_TYPE] == TYPE_MEDIA_STATUS:
             self._process_media_status(data)
@@ -340,7 +347,7 @@ class MediaController(BaseController):
 
         return False
 
-    def register_status_listener(self, listener):
+    def register_status_listener(self, listener: MediaStatusListener):
         """Register a listener for new media statuses. A new status will
         call listener.new_media_status(status)"""
         self._status_listeners.append(listener)
@@ -430,11 +437,11 @@ class MediaController(BaseController):
 
     def queue_next(self):
         """ Send the QUEUE_NEXT command. """
-        self._send_command({MESSAGE_TYPE: TYPE_QUEUE_NEXT})
+        self._send_command({MESSAGE_TYPE: TYPE_QUEUE_UPDATE, "jump": 1})
 
     def queue_prev(self):
         """ Send the QUEUE_PREV command. """
-        self._send_command({MESSAGE_TYPE: TYPE_QUEUE_PREV})
+        self._send_command({MESSAGE_TYPE: TYPE_QUEUE_UPDATE, "jump": -1})
 
     def enable_subtitle(self, track_id):
         """ Enable specific text track. """
@@ -480,18 +487,15 @@ class MediaController(BaseController):
             try:
                 listener.new_media_status(self.status)
             except Exception:  # pylint: disable=broad-except
-                _LOGGER.exception(
-                    "Exception thrown when calling media status " "callback"
-                )
+                _LOGGER.exception("Exception thrown when calling media status callback")
 
-    # pylint: disable=too-many-arguments
     def play_media(
         self,
         url,
         content_type,
         title=None,
         thumb=None,
-        current_time=0,
+        current_time=None,
         autoplay=True,
         stream_type=STREAM_TYPE_BUFFERED,
         metadata=None,
@@ -500,6 +504,7 @@ class MediaController(BaseController):
         subtitles_mime="text/vtt",
         subtitle_id=1,
         enqueue=False,
+        media_info=None,
     ):
         """
         Plays media on the Chromecast. Start default media receiver if not
@@ -510,8 +515,9 @@ class MediaController(BaseController):
         content_type: str - mime type. Example: 'video/mp4'.
         title: str - title of the media.
         thumb: str - thumbnail image url.
-        current_time: float - seconds from the beginning of the media
-            to start playback.
+        current_time: float - Seconds since beginning of content. If the content is
+            live content, and position is not specifed, the stream will start at the
+            live position
         autoplay: bool - whether the media will automatically play.
         stream_type: str - describes the type of media artifact as one of the
             following: "NONE", "BUFFERED", "LIVE".
@@ -520,12 +526,14 @@ class MediaController(BaseController):
         subtitles_mime: str - mimetype of subtitles.
         subtitle_id: int - id of subtitle to be loaded.
         enqueue: bool - if True, enqueue the media instead of play it.
+        media_info: dict - additional MediaInformation attributes not explicitly listed.
         metadata: dict - media metadata object, one of the following:
             GenericMediaMetadata, MovieMediaMetadata, TvShowMediaMetadata,
             MusicTrackMediaMetadata, PhotoMediaMetadata.
 
         Docs:
         https://developers.google.com/cast/docs/reference/messages#MediaData
+        https://developers.google.com/cast/docs/reference/web_receiver/cast.framework.messages.MediaInformation
         """
         # pylint: disable=too-many-locals
         def app_launched_callback():
@@ -544,6 +552,7 @@ class MediaController(BaseController):
                 subtitles_mime,
                 subtitle_id,
                 enqueue,
+                media_info,
             )
 
         receiver_ctrl = self._socket_client.receiver_controller
@@ -555,7 +564,7 @@ class MediaController(BaseController):
         content_type,
         title=None,
         thumb=None,
-        current_time=0,
+        current_time=None,
         autoplay=True,
         stream_type=STREAM_TYPE_BUFFERED,
         metadata=None,
@@ -564,13 +573,16 @@ class MediaController(BaseController):
         subtitles_mime="text/vtt",
         subtitle_id=1,
         enqueue=False,
+        media_info=None,
     ):
         # pylint: disable=too-many-locals
+        media_info = media_info or {}
         media = {
             "contentId": url,
             "streamType": stream_type,
             "contentType": content_type,
             "metadata": metadata or {},
+            **media_info,
         }
 
         if title:
@@ -620,7 +632,8 @@ class MediaController(BaseController):
                 "media": media,
                 MESSAGE_TYPE: TYPE_LOAD,
             }
-        msg["currentTime"] = current_time
+        if current_time is not None:
+            msg["currentTime"] = current_time
         msg["autoplay"] = autoplay
         msg["customData"] = {}
 
@@ -631,6 +644,6 @@ class MediaController(BaseController):
 
     def tear_down(self):
         """ Called when controller is destroyed. """
-        super(MediaController, self).tear_down()
+        super().tear_down()
 
         self._status_listeners[:] = []
