@@ -9,9 +9,8 @@ from .media import MediaController, STREAM_TYPE_BUFFERED, TYPE_LOAD, MESSAGE_TYP
 class YleAreenaController(MediaController):
     """ Controller to interact with Yle Areena app namespace. """
 
-    # pylint: disable=useless-super-delegation
     def __init__(self):
-        super(YleAreenaController, self).__init__()
+        super().__init__()
         self.app_id = APP_YLEAREENA
         self.supporting_app_id = APP_YLEAREENA
 
